@@ -275,6 +275,12 @@ cygwinify_path <- function(x = dir())
 #' newextension, possibly with a directory (see \code{include_dir} argument).
 #' \code{get_extension} returns a character vector of the third column.
 #' \code{recompose_path} returns a character vector of paths.
+#' @note Decomposing and then recomposing a path is usually equivalent to 
+#' standardizing that path (though slower).  That is, usually
+#' \code{recompose_path(decompose_path(x)) == standardize_path(x)}.
+#' One exception to this is when the directory of x is a symbolic link to 
+#' another directory.  In this case \code{decompose_path} will follow the
+#' link but \code{standardize_path} won't.
 #' @examples
 #' x <- c(
 #'   "somedir/foo.tgz",         # single extension
@@ -386,7 +392,7 @@ dir_copy <- function(...)
 #' slashes on Unix-based systems.
 #' @seealso \code{\link{is_windows_drive}}
 #' @examples
-#' get_drive(c("~", r_home(), temp_dir()))
+#' get_drive(c(".", "~", r_home(), temp_dir(), "\\\\foo/bar"))
 #' @importFrom assertive.reflection is_windows
 #' @importFrom utils head
 #' @export
@@ -720,7 +726,15 @@ recompose_path.decomposed_path <- function(x, ...)
     paste(x[not_missing, "filename"], x[not_missing, "extension"], sep = "."),
     x[not_missing, "filename"]
   )
-  path[not_missing] <- file.path(x[not_missing, "dirname"], base_x)
+  has_a_dir <- nzchar(as.character(x[not_missing, "dirname"]))
+  path[not_missing] <- ifelse(
+    has_a_dir,
+    file.path(x[not_missing, "dirname"], base_x),
+    base_x
+  )
+  # strip trailing slashes
+  path <- str_replace(path, "/?$", "")  
+  
   path
 }
 
@@ -829,6 +843,21 @@ split_path <- function(x = dir())
 #' named with the input file paths.
 #' @return A character vector of paths, pointing to the same locations as the
 #' input, but in a standardized form.
+#' @details \code{standardize_path} wraps \code{\link[base]{normalizePath}},
+#' providing additional tweaks to the output.
+#' \itemize{
+#' \item{Missing inputs always return \code{NA_character_}.}
+#' \item{Leading double back slashes are preserved under all OSes regardless of
+#' the values of \code{sep}.}
+#' \item{Leading double forward slashes are converted to double back slash under
+#' Windows (they are likely UNC paths), and a single forward slash under Unixes
+#' (they are likely absolute paths).}
+#' \item{Other back and forward slashes are replaced by \code{sep}.}
+#' \item{Paths are always made absolute.}
+#' \item{Trailing slashes are always stripped, except for root (\code{"/"}) and
+#' Windows drives (\code{"C:/"}, etc.).}
+#' \item{Windows drives are always capitalized.}
+#' }
 #' @seealso \code{\link[base]{normalizePath}}, \code{\link[base]{path.expand}},
 #' \code{\link[R.utils]{getAbsolutePath}}
 #' @examples
@@ -854,35 +883,68 @@ standardize_path <- function(x = dir(), sep = c("/", "\\"), include_names = TRUE
   x <- original_x <- coerce_to(x, "character")
   
   ok <- is_non_missing_nor_empty_character(x)
+
+  # normalizePath gives a silly result for "c:" under Windows, returning either
+  # r_home("bin") or temp_dir() or getwd()
+  # Convert it to "c:/" to fix.  Unclear if this affects only the OS drive
+  # or all mapped drives.  For safety, add a suffix to them all.
+  is_slashless_windows_drive <- str_detect(x[ok], "^[a-zA-Z]:$")
+  x[ok][is_slashless_windows_drive] <- paste0(x[ok][is_slashless_windows_drive], "/")
   
-  # standardize = expand + normalize
-  # normalizePath is uncomfortable with backslashes under Unix.
-  x[ok] <- str_replace_all(x[ok], "[/\\\\]", "/")
+  # Normalize, with smarter defaults, and returning NA for NA inputs.
   x[ok] <- ifelse(
     is.na(x[ok]),
     NA_character_,
     normalizePath(x[ok], "/", mustWork = FALSE)
   )
   
-  # again under Unix, normalizePath won't make path absolute
+  # Under Unix, normalizePath treats backslashes as characters in a filename.
+  # It's often more useful to assume these are Windows file separators, that is,
+  # We should replace all backslashes with (forward) slashes...
+  # except leading double backslashed which are considered as UNC paths.
+  # This code needs to come after the call to normalizePath, which has smart
+  # OS-dependent detection of UNC paths.  That is:
+  # Under Windows, normalizePath will have changed a leading // to \\.
+  # Under Unix a leading // is 
+  # - considered as an absolute path and changed to / if the path exists
+  # - left as // if it doesn't exist
+  # A leading \\ is preserved under all platforms.
+  x[ok] <- ifelse(
+    str_detect(x[ok], "^\\\\\\\\"), 
+    paste0("\\\\", str_replace_all(substring(x[ok], 3), fixed("\\"), "/")), 
+    str_replace_all(x[ok], fixed("\\"), "/")
+  )
+  
+  # Turn leading // into / when normalizePath forgot to.
   if(is_unix())
   {
-    x[ok] <- ifelse(
-      str_detect(x[ok], "^(/|[[:alpha:]]:)"),
-      x[ok], 
-      file.path(getwd(), x[ok], fsep = "/")
-    )
+    has_leading_double_slash <- str_detect(x[ok], "^/{2}")
+    x[ok][has_leading_double_slash] <- substring(x[ok][has_leading_double_slash], 2)
   }
   
-  # Under Windows, normalizePath prefixes UNC paths with backslashes rather than 
-  # forward slashes
-  if(is_windows())
+  # Again under Unix, normalizePath won't always make path absolute
+  if(is_unix())
   {
-    x[ok] <- str_replace(x[ok], "^\\\\\\\\", "//")
+    is_absolute <- str_detect(x[ok], "^([/\\\\]|[a-zA-Z]:)")
+    x[ok][!is_absolute] <- file.path(getwd(), x[ok], fsep = "/")
   }
   
-  # strip trailing slashes
-  x[ok] <- str_replace(x[ok], "/?$", "")  
+  # Strip trailing slashes, except if it's a root dir
+  # Root dir is either:
+  #   / (Unix root dir) 
+  #   an ascii letter then : then maybe a \ or / (Windows drive)
+  # Usually \ (Windows current drive) too, but this will have been replaced by
+  # normalizePath.
+  is_root <- str_detect(x[ok], "^(/|[a-zA-Z]:[/\\\\]?)$")
+  x[ok][!is_root] <- str_replace(x[ok][!is_root], "/?$", "")
+  # Root dirs should always end in /
+  needs_a_slash <- str_detect(x[ok][is_root], "[^/]$")  
+  x[ok][is_root][needs_a_slash] <- paste0(x[ok][is_root][needs_a_slash], "/")
+  
+  # Windows drive paths are sometimes but not always converted to upper case
+  # Make this consistently happen.
+  # At this point, the path should always start with a letter or a slash
+  x[ok] <- paste0(toupper(substring(x[ok], 1, 1)), substring(x[ok], 2))
   
   # Replace / with the chosen slash
   if(sep == "\\")
